
void CalibPID(){

  bool printGif = true;

  gStyle->SetOptStat(111111);
  gSystem->Load("libXMLParser.so");
  gSystem->Load("libanacore.so");
  gSystem->Load("libanabrips.so");

  TArtEventStore *estore = new TArtEventStore();

  TArtBigRIPSParameters *setup = new TArtBigRIPSParameters();
  setup->LoadParameter("db/PPAC.xml");
  setup->LoadParameter("db/Plastic.xml");
  setup->LoadParameter("db/IC.xml");

  TArtCalibPID *mycalib= new TArtCalibPID(setup);

  TArtRawEventObject *rawevent = new TArtRawEventObject();
  estore->Initialize(rawevent); 
  estore->Open("test.ridf");
 
  /* prepare histograms */
  char dummy[64];
  TH1 * hICdE[4];
  TH1 * hPlasticTiming[12];
  TH2 * hPPACPos[36];  

  Int_t NumIC = mycalib->GetNumIC();
  Int_t NumPlastic = mycalib->GetNumPlastic();
  Int_t NumPPAC = mycalib->GetNumPPAC();
  for(int i=0;i<NumIC;i++) {
    TArtIC * myic = mycalib->GetIC(i);
    sprintf(dummy,"ic_de_%d",myic->GetID());
    hICdE[i] = new TH1F(dummy,myic->GetDetectorName()->Data(),100,0,1000);
  }
  for(int i=0;i<NumPlastic;i++) {
    TArtPlastic * mypla = mycalib->GetPlastic(i);
    sprintf(dummy,"plastic_t_%d",mypla->GetID());
    hPlasticTiming[i] = new TH1F(dummy,mypla->GetDetectorName()->Data(),100,0,100);
  }
  for(int i=0;i<NumPPAC;i++) {
    TArtPPAC * myppac = mycalib->GetPPAC(i);
    sprintf(dummy,"ppac_pos_%d",myppac->GetID());
    hPPACPos[i] = new TH2F(dummy,myppac->GetDetectorName()->Data(),100,-400,400,100,-400,400);
  }
 
  std::cout << "start to scan" << std::endl;

  int neve = 0;
  while(estore->GetNextEvent() && neve<10000){
    mycalib->ReconstructData(rawevent);

    for(int i=0;i<NumIC;i++) 
      hICdE[i]->Fill(mycalib->GetIC(i)->GetCalMeVSqSum());
    for(int i=0;i<NumPlastic;i++) if(mycalib->GetPlastic(i)->IsFired()) 
      hPlasticTiming[i]->Fill(mycalib->GetPlastic(i)->GetTime());
    for(int i=0;i<NumPPAC;i++) if(mycalib->GetPPAC(i)->IsFired()) 
      hPPACPos[i]->Fill(mycalib->GetPPAC(i)->GetX(),mycalib->GetPPAC(i)->GetY());

    mycalib->ClearData();
    rawevent->Clear();
    neve ++;
  }

  TCanvas *cic = new TCanvas("cic","cic",1600,1200);
  cic->Divide(2,2);
  for(int i=0;i<NumIC;i++) {
    cic->cd(i+1); hICdE[i]->Draw();
  }
  if(printGif)cic->Print("cic.gif");

  TCanvas *cpla = new TCanvas("cpla","cpla",1600,1200);
  cpla->Divide(4,3);
  for(int i=0;i<NumPlastic;i++) {
    cpla->cd(i+1); hPlasticTiming[i]->Draw();
  }
  if(printGif)cpla->Print("cpla.gif");

  TCanvas *cppac = new TCanvas("cppac","cppac",1600,1200);
  cppac->Divide(6,6);
  for(int i=0;i<NumPPAC;i++) {
    cppac->cd(i+1); hPPACPos[i]->Draw("colz");
  }
  if(printGif)cppac->Print("cppac.gif");

  return;

}
