// BigRIPS/ZeroDegree reconstruction class
// 

#ifndef _TARTPID_H_
#define _TARTPID_H_

class TArtCalibPID;
class TArtFocalPlane;
class TArtFocalPlaneArray;
class TArtFocalPlanePara;
class TArtRIPS;
class TArtRIPSArray;
class TArtRIPSPara;
class TArtTOF;
class TArtTOFArray;
class TArtTOFPara;
class TArtBeam;
class TArtBeamArray;
class TArtBeamPara;

class TArtPID  {

 public:

  TArtPID();
  TArtPID(TArtBigRIPSParameters *s);
  TArtPID(TArtBigRIPSParameters *s, TArtCalibPID *calib);
  virtual ~TArtPID();

  TArtFocalPlaneArray* GetFocalPlane(){return fpl_array;}
  TArtFocalPlane*      GetFocalPlane(int id);
  TArtFocalPlane*      GetFocalPlane(TArtFocalPlanePara *fp);
  Int_t                GetNumFocalPlane();
  TArtRIPSArray*       GetRIPS(){return rips_array;}
  TArtRIPS*            GetRIPS(int id);
  TArtRIPS*            GetRIPS(TArtRIPSPara *rp);
  Int_t                GetNumRIPS();
  TArtTOFArray*        GetTOF(){return tof_array;}
  TArtTOF*             GetTOF(int id);
  TArtTOF*             GetTOF(TArtTOFPara *tp);
  Int_t                GetNumTOF();
  TArtBeamArray*       GetBeam(){return beam_array;}
  TArtBeam*            GetBeam(int id);
  TArtBeam*            GetBeam(TArtBeamPara *bp);
  Int_t                GetNumBeam();

  void      ClearData();
  void      LoadDetectorMap(TArtCalibPID *);
  void      ReconstructData();
  void      ReconstructData(TArtCalibPID *);
 
  Bool_t    IsReconstructed(){ return fReconstructed; }

 private:

  void      Initialize();
  void      Initialize(TArtCalibPID *);
  Bool_t    fReconstructed;
  Bool_t    fDetectorMapLoaded;

  // setup parameter bank
  TArtBigRIPSParameters *setup;
  TArtFocalPlaneArray * fpl_array;
  TArtRIPSArray * rips_array;
  TArtTOFArray * tof_array;
  TArtBeamArray * beam_array;

  ClassDef(TArtPID,1);

 };
#endif
