#include "TArtPPAC.h" 
#include "TArtPPACPara.h" 
#include "TArtCore.h" 
#include "TArtMapper.h" 
#include "TArtBigRIPSParameters.h"

#include <TRandom.h>
#include <TList.h>
#include <TH2.h>

//__________________________________________________________
void TArtPPACArray::LoadSetup(TArtBigRIPSParameters *para)
{

  Clear();
  TList *listOfPPACPara = (TList *)para->GetListOfPPACPara();
  TIter next(listOfPPACPara);
  TArtPPACPara *p;
  while(p = (TArtPPACPara*)next()) 
    Add(new TArtPPAC(p));

}

//__________________________________________________________
void TArtPPACArray::LoadRawDataMap(TArtMapper *mapper) 
{
  for(int i=0;i<GetEntries();i++)
    ((TArtPPAC*)At(i))->LoadRawDataMap(mapper);
}

//__________________________________________________________
void TArtPPACArray::ClearData()
{
  for(int i=0;i<GetEntries();i++)
    ((TArtPPAC*)At(i))->ClearData();
}

//__________________________________________________________
void TArtPPACArray::ReconstructData()
{
  for(int i=0;i<GetEntries();i++)
    ((TArtPPAC*)At(i))->ReconstructData();
}

//__________________________________________________________
TArtPPAC::TArtPPAC(TArtPPACPara *p) 
  :fReconstructed(0), para(p)
{
}

//__________________________________________________________
TArtPPAC::~TArtPPAC() {
} 

//__________________________________________________________
void TArtPPAC::LoadRawDataMap(TArtMapper *mapper) {

  TArtCore::Info(__FILE__,"mapping: TX1 of %s is %s", GetDetectorName()->Data(), ((TArtRIDFMap *)para->GetTX1Map())->GetMapInfo());
  mapper->LoadRecoConfig(para->GetTX1Map(), &fTX1Raw);
  TArtCore::Info(__FILE__,"mapping: TX2 of %s is %s", GetDetectorName()->Data(), ((TArtRIDFMap *)para->GetTX2Map())->GetMapInfo());
  mapper->LoadRecoConfig(para->GetTX2Map(), &fTX2Raw);
  TArtCore::Info(__FILE__,"mapping: TY1 of %s is %s", GetDetectorName()->Data(), ((TArtRIDFMap *)para->GetTY1Map())->GetMapInfo());
  mapper->LoadRecoConfig(para->GetTY1Map(), &fTY1Raw);
  TArtCore::Info(__FILE__,"mapping: TY2 of %s is %s", GetDetectorName()->Data(), ((TArtRIDFMap *)para->GetTY2Map())->GetMapInfo());
  mapper->LoadRecoConfig(para->GetTY2Map(), &fTY2Raw);
  TArtCore::Info(__FILE__,"mapping: TA of %s is %s", GetDetectorName()->Data(), ((TArtRIDFMap *)para->GetTAMap())->GetMapInfo());
  mapper->LoadRecoConfig(para->GetTAMap() , &fTARaw);

}

//__________________________________________________________
void TArtPPAC::ClearData() {

  fTX1Raw = 0;
  fTX2Raw = 0;
  fTY1Raw = 0;
  fTY2Raw = 0;
  fTARaw   = 0;
  fX      = -999.;
  fY      = -999.;
  fFiredX = false;
  fFiredY = false;
  fReconstructed = false;

}

//__________________________________________________________
void TArtPPAC::ReconstructData() {

  static TRandom *r = new TRandom();

  fX      = -999.;
  fY      = -999.;
  fFiredX = false;     
  fFiredY = false;
 
  double  fTSumX = -9999.; 
  double  fTSumY = -9999.; 
  double  fTDiffX = -9999.;
  double  fTDiffY = -9999.;

  float random = r->Rndm();
  double fTX1 = (fTX1Raw - 0.5 + random) * para->GetCh2NsX1();
  random = r->Rndm();
  double fTX2 = (fTX2Raw - 0.5 + random) * para->GetCh2NsX2();
  random = r->Rndm();
  double fTY1 = (fTY1Raw - 0.5 + random) * para->GetCh2NsY1();
  random = r->Rndm();
  double fTY2 = (fTY2Raw - 0.5 + random) * para->GetCh2NsY2();
  random = r->Rndm();
  double fTA  = (fTARaw - 0.5 + random) * para->GetCh2NsA();
   
  if(fTX1Raw >0 && fTX1Raw < 4000. && fTX2Raw >0 && fTX2Raw < 4000. ) {
    fTSumX = fTX1 + fTX2 - 2*fTA;
    fTDiffX = fTX1 - fTX2;
  }
  if(fTY1Raw >0 && fTY1Raw < 4000. && fTY2Raw >0 && fTY2Raw < 4000. ) {
    fTSumY = fTY1 + fTY2 - 2*fTA;
    fTDiffY = fTY1 - fTY2;
  }

  if((para->GetTXSumMin() >= para->GetTXSumMax()) || 
     (fTSumX >= para->GetTXSumMin() && fTSumX <= para->GetTXSumMax()) ){  
    fTDiffX = fTDiffX - para->GetXDTimeOffset();
    fX = fTDiffX * para->GetXFactor() / 2.;
    //    Double_t fXX = fX;
    fX -= para->GetXOffset();
    fX -= para->GetXPosOffset();
    fX = -fX; // definition for optics only for X
    fFiredX = true; // The ppac "really fired correctly"
    //    TArtCore::Info(__FILE__,"%s: RawTX1:%d, TXdiff:%f, fXX:%f,  Ch2NsX1:%f, Ch2NsX2:%f, XDTimeOffset:%f, XFactor:%f, XOffset:%f, XPosOffset:%f, X:%f", GetDetectorName()->Data(), fTX1Raw, fTDiffX, fXX, para->GetCh2NsX1(), para->GetCh2NsX2(), para->GetXDTimeOffset(), para->GetXFactor(), para->GetXOffset(), para->GetXPosOffset(),fX);
  }
  if((para->GetTYSumMin() >= para->GetTYSumMax()) || 
     (fTSumY >= para->GetTYSumMin() && fTSumY <= para->GetTYSumMax()) ){  
    fTDiffY = fTDiffY - para->GetYDTimeOffset();
    fY = fTDiffY * para->GetYFactor() / 2.;
    fY -= para->GetYOffset();
    fY -= para->GetYPosOffset();
    fFiredY = true; // The ppac "really fired correctly"
  }

  fReconstructed = true;

}

//__________________________________________________________
Int_t TArtPPAC::GetID() {
  return para->GetID();
}
//__________________________________________________________
Int_t TArtPPAC::GetFpl() {
  return para->GetFpl();
}
//__________________________________________________________
Double_t TArtPPAC::GetXZPos() {
  return para->GetXZPos();
}
//__________________________________________________________
Double_t TArtPPAC::GetYZPos() {
  return para->GetYZPos();
}
//__________________________________________________________
TString * TArtPPAC::GetDetectorName() {
  return (TString *)para->GetDetectorName();
}
