#ifndef _TARTDALIPARA_H_
#define _TARTDALIPARA_H_

#include <Riostream.h>

#include "TArtMap.h"
#include "segidlist.h"

class TArtDALIPara : public TNamed {

 public:
  TArtDALIPara(Int_t i, TString n, Int_t f, Int_t l, Double_t tc, Double_t to, Double_t qc, Double_t qp, Double_t th) :
    id(i), detname(n), fpl(f), layer(l), tcal(tc), toff(to), qcal(qc), qped(qp), theta(th){ }
  ~TArtDALIPara() { }

  void SetMap(Int_t tgeo, Int_t tch, Int_t qgeo, Int_t qch) {
    map_adc.SetMap(fpl,DALIA,qgeo,qch); 
    cout << "ADC is mapped to " << map_adc.GetMapInfo() << " " << map_adc.GetName() << endl;
    map_tdc.SetMap(fpl,DALIT,tgeo,tch); 
    cout << "TDC is mapped to " << map_tdc.GetMapInfo() << " " << map_tdc.GetName() << endl;
  }

  Int_t GetID() const { return id; }
  TString * GetDetectorName() { return &detname; }
  Int_t GetFpl() const { return fpl; }
  Int_t GetNaILayer() const { return layer; }
  Double_t GetTCal() const { return tcal; } 
  Double_t GetTOffset() const { return toff; } 
  Double_t GetQCal() const { return qcal; } 
  Double_t GetQPed() const { return qped; } 
  Double_t GetPos(int i) const { return pos[i]; } 
  Double_t GetNaITheta() const { return theta; }

  TArtRIDFMap * GetTDCMap() { return &map_tdc;  }
  TArtRIDFMap * GetADCMap() { return &map_adc;  }

  friend ostream & operator << (ostream& out, const TArtDALIPara& p) {
    out << "ID: " << p.id << endl;
    out << "DetectorName: " << p.detname << endl;
    out << "Focal Plane: " << p.fpl << endl;
    out << "NaI Layer: " << p.layer << endl;
    out << "Time Calib: " << p.tcal << endl;
    out << "Time Offset: " << p.toff << endl;
    out << "QDC Calib: " << p.qcal << endl;
    out << "QDC pedestal: " << p.tcal << endl;
    out << "Position: " <<p.pos[0]<< " " <<p.pos[1]<< " " <<p.pos[2]<< endl;
    out << "Theta: "  << p.theta << endl;
    out << "Map of ADC: "  << p.map_adc << endl;
    out << "Map of TDC: "  << p.map_tdc << endl;
    out << endl;
    return out;
  }

 private:
  Int_t     id;
  TString   detname;
  Int_t     fpl;
  Int_t     layer;
  Double_t  tcal;
  Double_t  toff;
  Double_t  qcal;
  Double_t  qped;
  Double_t  pos[3]; // position in mm
  Double_t  theta; // angle w.r.t. target
  TArtRIDFMap map_tdc;
  TArtRIDFMap map_adc;

  ClassDef(TArtDALIPara,1);

};

#endif
